local actor = require("luactor")
do
  local _base_0 = {
    offer = function(self, value)
      self.last = self.last + 1
      self.items[self.last] = value
    end,
    poll = function(self)
      if #self.items == 0 then
        self.first = 1
        self.last = 0
        return nil
      end
      local value = self.items[self.first]
      self.items[self.first] = nil
      self.first = self.first + 1
      return value
    end
  }
  _base_0.__index = _base_0
  local _class_0 = setmetatable({
    __init = function(self)
      self.first = 1
      self.last = 0
      self.items = { }
    end,
    __base = _base_0,
    __name = "Queue"
  }, {
    __index = _base_0,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  Queue = _class_0
end
local __msg_queue = Queue()
local mixin
mixin = function(self, cls, ...)
  local _mixin = self.loaded_mixins[cls]
  local refcount = 0
  if _mixin then
    refcount = _mixin.refcount
  end
  if _mixin then
    _mixin.refcount = _mixin.refcount + 1
  else
    self.loaded_mixins[cls] = {
      cls = cls,
      refcount = 1
    }
    cls.__init(self, ...)
    for key, val in pairs(cls.__base) do
      local new_method_name = cls.__name .. key
      if not key:match("^__") then
        self[new_method_name] = val
        self:add_handler(key, val)
      end
    end
  end
  if cls.needs ~= nil then
    for _, val in pairs(cls.needs) do
      local val_cls = _G[val]
      if val_cls == nil then
        error("Does not exist: " .. val)
      end
      mixin(self, val_cls)
    end
  end
  if cls.mixed_in then
    return cls.mixed_in(self, cls.__class.__name, '_', _mixin == nil)
  end
end
local mixout
mixout = function(self, cls, ...)
  local _mixin = self.loaded_mixins[cls]
  if _mixin == nil then
    return 
  end
  _mixin.refcount = _mixin.refcount - 1
  if cls.needs ~= nil then
    for _, val in pairs(cls.needs) do
      local val_cls = _G[val]
      if val_cls == nil then
        error("Does not exist: " .. val)
      end
      mixout(self, val_cls)
    end
  end
  if 0 < _mixin.refcount then
    return 
  end
  if cls.mixed_out then
    cls.mixed_out(self, cls.__class.__name, '_')
  end
  for key, val in pairs(cls.__base) do
    local new_method_name = cls.__name .. key
    if not key:match("^__") then
      self[new_method_name] = nil
      self:remove_handler(key, val)
    end
  end
  self.loaded_mixins[cls] = nil
end
local __components = { }
local __component_methods = { }
do
  local _base_0 = { }
  _base_0.__index = _base_0
  local _class_0 = setmetatable({
    __init = function() end,
    __base = _base_0,
    __name = "Component"
  }, {
    __index = _base_0,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  local self = _class_0
  self.__inherited = function(self, child)
    __components[child.__name] = child
    for _, method in pairs(child.__base) do
      __component_methods[method] = child
    end
  end
  Component = _class_0
end
local object_count = 0
objects = { }
do
  local _base_0 = {
    add_handler = function(self, method_name, method)
      if not self.handlers[method_name] then
        self.handlers[method_name] = { }
      end
      return table.insert(self.handlers[method_name], method)
    end,
    remove_handler = function(self, method_name, method)
      if self.handlers[method_name] then
        for key, existing_method in pairs(self.handlers[method_name]) do
          if existing_method == method then
            table.remove(self.handlers[method_name], key)
          end
        end
      end
    end,
    _start = function(self) end,
    _think = function(self) end,
    _mixin = function(self, cls)
      return mixin(self, cls)
    end,
    mixin = function(self, class_name, sender)
      return self:_mixin(_G[class_name])
    end,
    mixin_if_not_mixed_in = function(self, class_name, sender)
      if self:is_mixed_in(class_name) then
        return 
      end
      return self:_mixin(_G[class_name])
    end,
    mixout = function(self, class_name, sender)
      local cls = _G[class_name]
      if self.loaded_mixins[cls] == nil then
        return 
      end
      mixout(self, cls)
      local is_mixed_out = self.loaded_mixins[cls] == nil
      if is_mixed_out then
        return self:send(sender, 'was_mixed_out', class_name)
      end
    end,
    remove = function(self, msg, sender)
      for key, obj in pairs(objects) do
        if obj == self then
          table.remove(objects, key)
        end
      end
    end,
    die = function(self, class_name, sender)
      self.died = true
    end,
    is_mixed_in = function(self, class_name)
      local cls = _G[class_name]
      return self.loaded_mixins[cls] ~= nil
    end,
    send = function(self, sendee, msg_name, msg_data)
      return __msg_queue:offer({
        self.id,
        sendee,
        msg_name,
        msg_data
      })
    end,
    set_fields = function(self, msg, sender)
      if msg == nil then
        return 
      end
      for field, value in pairs(msg) do
        self[field] = value
      end
    end
  }
  _base_0.__index = _base_0
  local _class_0 = setmetatable({
    __init = function(self)
      self.died = false
      self.loaded_mixins = { }
      self.handlers = { }
      self.id = tostring(object_count)
      object_count = object_count + 1
      self:mixins()
      objects[self.id] = self
      self:add_handler("mixin", Object.mixin)
      self:add_handler("mixout", Object.mixout)
      self:add_handler("remove", Object.remove)
      self:add_handler("die", Object.die)
      self:add_handler("set_fields", Object.set_fields)
      self:add_handler("mixin_if_not_mixed_in", Object.mixin_if_not_mixed_in)
      self:_start()
      return self:send(self.id, 'init')
    end,
    __base = _base_0,
    __name = "Object"
  }, {
    __index = _base_0,
    __call = function(cls, ...)
      local _self_0 = setmetatable({}, _base_0)
      cls.__init(_self_0, ...)
      return _self_0
    end
  })
  _base_0.__class = _class_0
  Object = _class_0
end
poll_messages = function()
  local items = __msg_queue.items
  __msg_queue.items = { }
  __msg_queue.first = 1
  __msg_queue.last = 0
  for _, msg in pairs(items) do
    local sendor = msg[1]
    local sendee = msg[2]
    local msg_name = msg[3]
    local msg_data = msg[4]
    local sendee_object = objects[sendee]
    if not sendee_object then
      print("FAILED to send to object")
      return 
    end
    local handlers = sendee_object.handlers[msg_name]
    if handlers then
      for _, method in pairs(handlers) do
        method(sendee_object, msg_data, sendor)
      end
    end
  end
end
